<?php

namespace Drupal\ab_age_gate\EventSubscriber;

use Drupal\ab_age_gate\AgeGateStatistics;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Render\AttachmentsInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

class AgeGateSubscriber implements EventSubscriberInterface {

  /**
   * Current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  protected $request;

  /**
   * Configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\ab_age_gate\AgeGateStatistics $statisticsService;
   */
  protected $statisticsService;

  /**
   * MySimpleCookie constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request stack service.
   */
  public function __construct(RequestStack $request_stack, ConfigFactoryInterface $config_factory, AgeGateStatistics $statisticsService) {
    $this->request = $request_stack->getCurrentRequest();
    $this->configFactory = $config_factory;
    $this->statisticsService = $statisticsService;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [KernelEvents::RESPONSE => ['onResponse', 30]];
  }

  public function onResponse(ResponseEvent $event) {
    // Only act upon the master request and not sub-requests.
    if ($event->isMasterRequest()) {
      $config = $this->configFactory->get('ab_age_gate.settings');

      // Process ignore pages settings.
      $ignore_pages = $config->get('ignore_pages');
      $ignore_pages = explode(PHP_EOL, $ignore_pages);
      foreach ($ignore_pages as $ignore_page) {
        if (!empty($ignore_page)) {
          if ($ignore_page == '<front>') {
            $base_url = \Drupal::urlGenerator()
              ->generateFromRoute('<front>', [], ['absolute' => TRUE]);
            if ($base_url === $this->request->getUri()) {
              return;
            }
          }
          if (str_contains($this->request->getUri(), $ignore_page)) {
            return;
          }
        }
      }

      // Early exit if admin pages.
      $excluded_path = [
        'admin',
        'node/add',
        'translations/add'
      ];
      foreach ($excluded_path as $path) {
        if (str_contains($this->request->getUri(), $path)) {
          return;
        }
      }

      $response = $event->getResponse();
      // Only act if the response is one that is able to have attachments.
      if (!$this->request->cookies->get('agegate') && $response instanceof AttachmentsInterface) {
        // Get any existing attachments.
        $attachments = $response->getAttachments();
        // Set library and settings.
        $attachments['drupalSettings']['ageGateSettings'] = $config->getRawData();
        $attachments['library'][] = 'ab_age_gate/age_gate';
        // Set the updated array back on the object.
        $response->setAttachments($attachments);
      }
    }
  }

}
